'use strict';

var chalk = require('chalk');
var child_process = require('child_process');
var di = require('di');
var gspdata = require('./util/gspdata');
var file = require('./file');
var fs = require('fs-extra');
var http = require('http');
var mine = require('mime');
var path = require('path');
var url = require('url');
var watch = require('./watch');

var $repoLocation = {};
var $repoMapping = {};

var start = function start(options, watch, file) {

    if (!fs.existsSync(options.cwd + '/.gsp_workspace')) {
        console.log(chalk.red('Please run on the directory containing git repositories.'));
        return;
    }

    options.port = options.port || 7070;

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = fs.readdirSync(options.cwd)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var repo = _step.value;

            var gspconfig = path.join(options.cwd, repo, '.gspconfig');
            if (!fs.existsSync(gspconfig)) {
                continue;
            }
            gspconfig = fs.readJSONSync(gspconfig, { throws: false }) || {};
            $repoLocation[repo] = path.join(options.cwd, repo);
            if (gspconfig.mapping_dir) {
                $repoMapping[gspconfig.mapping_dir] = repo;
            }
            var hookdir = path.join(options.cwd, repo, '.git/hooks/pre-commit');
            fs.outputFileSync(hookdir, fs.readFileSync(path.join(__dirname, '../scripts/pre-commit')));
            fs.chmodSync(hookdir, '751');
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator['return']) {
                _iterator['return']();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    gspdata.set('repositories', $repoLocation);

    var server = http.createServer();
    server.on('listening', function (err) {
        if (err) {
            console.log(chalk.red('Gsp simulator failed to start'));
        } else {
            console.log('Gsp simulator started, listening on port %s', options.port);
            if (options.livereload) {
                watch({
                    base: options.cwd,
                    livereload: true
                });
            }
        }
    });
    server.on('request', function (request, response) {
        var pathname = url.parse(request.url).pathname;
        var output = function output(code, data) {
            if (data) {
                data = new Buffer(data);
                response.writeHeader(code, { 'content-type': mine.lookup(pathname), 'content-length': data.length });
                response.end(data);
            } else {
                response.end();
            }
        };
        var repoId = undefined,
            filename = undefined;
        Object.keys($repoMapping).some(function (mappingDir) {
            var mappingDirfixed = mappingDir.replace(/^\/*/, '/').replace(/\/*$/, '/');
            if (pathname.indexOf(mappingDirfixed) === 0) {
                repoId = $repoMapping[mappingDir];
                filename = pathname.slice(mappingDirfixed.length);
                return true;
            }
        });

        if (repoId) {
            var workdir = $repoLocation[repoId];
            var config = fs.readJSONSync(path.join(workdir, '.gspconfig'), { throws: false }) || {};
            filename = path.join(config.publish_dir || '', filename);
            file.set('workdir', workdir);
            file.set('filename', filename);
            file.set('config', config);
            file.read(function (err, filedata) {
                console.log('GET: ' + pathname);
                if (err) {
                    console.log(chalk.red(err.message));
                    if (filedata) {
                        output(502);
                    } else {
                        output(404);
                    }
                } else {
                    output(200, filedata);
                }
            });
        } else {
            console.log('GET: ' + pathname);
            console.log(chalk.red('Git repository lacks.'));
            output(404);
        }
    });
    server.listen(options.port);
};

exports.start = function (options) {
    options.cwd = options.cwd || process.cwd();
    process.chdir(options.cwd);
    if (options.daemon) {
        delete options.daemon;
        var args = process.argv.filter(function (arg) {
            return arg !== '-d' && arg !== '--daemon';
        });
        args.shift();
        var out = fs.openSync(path.join(options.cwd, 'out.log'), 'a');
        child_process.spawn(args.shift(), args, {
            detached: true,
            stdio: ['ignore', out, out]
        });
    } else {
        var injector = new di.Injector([{
            options: ['value', options],
            watch: ['value', watch.watch],
            file: ['type', file.Concat]
        }]);
        injector.invoke(start);
    }
};
